@@ -406,7 +406,9 @@ static bool auth_write_handler(struct l_io *io, void *user_data)
 	if (!len)
 		return false;
 
-	written = send(fd, classic->auth_command, len, 0);
+	written = TEMP_FAILURE_RETRY(send(fd, classic->auth_command, len, 0));
+	if (written < 0)
+		return false;
 
 	l_util_hexdump(false, classic->auth_command, written,
 					dbus->debug_handler, dbus->debug_data);
@@ -456,11 +458,13 @@ static bool auth_read_handler(struct l_io *io, void *user_data)
 	offset = 0;
 
 	while (1) {
-		len = recv(fd, ptr + offset, sizeof(buffer) - offset,
-				MSG_DONTWAIT);
-		if (len < 1) {
-			if (len == -1 && errno == EINTR)
-				continue;
+		len = TEMP_FAILURE_RETRY(recv(fd, ptr + offset,
+						sizeof(buffer) - offset,
+						MSG_DONTWAIT));
+		if (len < 0) {
+			if (errno != EAGAIN)
+				return false;
+
 			break;
 		}
 
@@ -575,7 +579,7 @@ static void dbus_init(struct l_dbus *dbus, int fd)
 static void classic_free(struct l_dbus *dbus)
 {
 	struct l_dbus_classic *classic =
-		container_of(dbus, struct l_dbus_classic, super);
+		l_container_of(dbus, struct l_dbus_classic, super);
 	unsigned int i;
 
 	for (i = 0; i < classic->num_fds; i++)
@@ -594,7 +598,7 @@ static bool classic_send_message(struct l_dbus *dbus,
 	struct msghdr msg;
 	struct iovec iov[2], *iovpos;
 	ssize_t r;
-	int *fds;
+	int *fds = NULL;
 	uint32_t num_fds = 0;
 	struct cmsghdr *cmsg;
 	int iovlen;
@@ -625,13 +629,9 @@ static bool classic_send_message(struct l_dbus *dbus,
 			memcpy(CMSG_DATA(cmsg), fds, num_fds * sizeof(int));
 		}
 
-		r = sendmsg(fd, &msg, 0);
-		if (r <= 0) {
-			if (r < 0 && errno != EINTR)
-				return false;
-
-			continue;
-		}
+		r = TEMP_FAILURE_RETRY(sendmsg(fd, &msg, 0));
+		if (r < 0)
+			return false;
 
 		while ((size_t) r >= iovpos->iov_len) {
 			r -= iovpos->iov_len;
@@ -658,7 +658,7 @@ static bool classic_send_message(struct l_dbus *dbus,
 static struct l_dbus_message *classic_recv_message(struct l_dbus *dbus)
 {
 	struct l_dbus_classic *classic =
-		container_of(dbus, struct l_dbus_classic, super);
+		l_container_of(dbus, struct l_dbus_classic, super);
 	int fd = l_io_get_fd(dbus->io);
 	struct dbus_header hdr;
 	struct msghdr msg;
@@ -702,13 +702,10 @@ static struct l_dbus_message *classic_recv_message(struct l_dbus *dbus)
 		msg.msg_control = &fd_buf;
 		msg.msg_controllen = sizeof(fd_buf);
 
-		r = recvmsg(fd, &msg, MSG_CMSG_CLOEXEC | MSG_WAITALL);
-		if (r < 0) {
-			if (errno != EINTR)
-				goto cmsg_fail;
-
-			continue;
-		}
+		r = TEMP_FAILURE_RETRY(recvmsg(fd, &msg,
+					MSG_CMSG_CLOEXEC | MSG_WAITALL));
+		if (r < 0)
+			goto cmsg_fail;
 
 		for (cmsg = CMSG_FIRSTHDR(&msg); cmsg;
 				cmsg = CMSG_NXTHDR(&msg, cmsg)) {
@@ -812,7 +809,7 @@ static bool classic_add_match(struct l_dbus *dbus, unsigned int id,
 				int rule_len)
 {
 	struct l_dbus_classic *classic =
-		container_of(dbus, struct l_dbus_classic, super);
+		l_container_of(dbus, struct l_dbus_classic, super);
 	char *match_str;
 	struct l_dbus_message *message;
 
@@ -836,7 +833,7 @@ static bool classic_add_match(struct l_dbus *dbus, unsigned int id,
 static bool classic_remove_match(struct l_dbus *dbus, unsigned int id)
 {
 	struct l_dbus_classic *classic =
-		container_of(dbus, struct l_dbus_classic, super);
+		l_container_of(dbus, struct l_dbus_classic, super);
 	char *match_str = l_hashmap_remove(classic->match_strings,
 						L_UINT_TO_PTR(id));
 	struct l_dbus_message *message;
@@ -1054,7 +1051,7 @@ static struct l_dbus *setup_dbus1(int fd, const char *guid)
 		ptr += sprintf(ptr, "%02x", uid[i]);
 
 	/* Send special credentials-passing nul byte */
-	written = send(fd, &creds, 1, 0);
+	written = TEMP_FAILURE_RETRY(send(fd, &creds, 1, 0));
 	if (written < 1) {
 		close(fd);
 		return NULL;
@@ -1131,7 +1128,7 @@ static struct l_dbus *setup_unix(char *params)
 		}
 
 		addr.sun_path[0] = '\0';
-		strncpy(addr.sun_path + 1, path, len);
+		strncpy(addr.sun_path + 1, path, sizeof(addr.sun_path) - 2);
 		len++;
 	} else {
 		if (len > sizeof(addr.sun_path)) {
@@ -1139,7 +1136,7 @@ static struct l_dbus *setup_unix(char *params)
 			return NULL;
 		}
 
-		strncpy(addr.sun_path, path, len);
+		strncpy(addr.sun_path, path, sizeof(addr.sun_path) - 1);
 	}
 
 	if (connect(fd, (struct sockaddr *) &addr,
@@ -1289,7 +1286,7 @@ LIB_EXPORT bool l_dbus_set_debug(struct l_dbus *dbus,
 	dbus->debug_destroy = destroy;
 	dbus->debug_data = user_data;
 
-	//l_io_set_debug(dbus->io, function, user_data, NULL);
+	/* l_io_set_debug(dbus->io, function, user_data, NULL); */
 
 	return true;
 }
