@@ -57,3 +57,6 @@
 #include <ell/dbus-client.h>
 #include <ell/dhcp.h>
 #include <ell/cert.h>
+#include <ell/ecc.h>
+#include <ell/ecdh.h>
+#include <ell/time.h>
